package cc.fascinated.services;

import cc.fascinated.exception.impl.BadRequestException;
import cc.fascinated.model.user.User;
import cc.fascinated.repository.mongo.UserRepository;
import lombok.NonNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.UUID;

/**
 * @author Fascinated (fascinated7)
 */
@Service
public class UserService {
    /**
     * The user repository to use
     */
    @NonNull private final UserRepository userRepository;

    @Autowired
    public UserService(@NonNull UserRepository userRepository) {
        this.userRepository = userRepository;
    }

    /**
     * Gets a user by their id
     *
     * @param steamId the id of the user's steam profile
     * @return the user
     * @throws BadRequestException if the user is not found
     */
    public User getUser(String steamId) {
        if (!this.isValidSteamId(steamId)) {
            throw new BadRequestException("Invalid steam id");
        }

        Optional<User> userOptional = this.userRepository.findBySteamId(steamId);
        if (userOptional.isEmpty()) {
            // todo: check the steam API to see if the user exists
            User user = new User(UUID.randomUUID());
            user.setSteamId(steamId);
            return userRepository.save(user);
        }
        return userOptional.get();
    }

    /**
     * Saves a user to the database
     *
     * @param user the user to save
     */
    public void saveUser(User user) {
        this.userRepository.save(user);
    }

    /**
     * Gets all users in the database
     *
     * @return all users
     */
    public List<User> getUsers(boolean steamIdsOnly) {
        if (steamIdsOnly) {
            return this.userRepository.fetchOnlySteamIds();
        }
        return this.userRepository.findAll();
    }

    /**
     * Validates a steam id
     *
     * @param steamId the steam id to validate
     * @return if the steam id is valid
     */
    public boolean isValidSteamId(String steamId) {
        return steamId != null && steamId.length() == 17;
    }
}
